@@ -5,20 +5,31 @@
 #' @importFrom rgl plot3d
 #' @importFrom rgl text3d
 #' @importFrom rgl segments3d
+#' @importFrom rgl par3d
+#' @importFrom rgl bgplot3d
 #' @importFrom grDevices rainbow
+#' @importFrom grDevices heat.colors
+#' @importFrom grDevices terrain.colors
+#' @importFrom grDevices topo.colors
+#' @importFrom grDevices cm.colors
+#' @importFrom grDevices as.raster
+#' @importFrom graphics rasterImage
+#' @importFrom grDevices colorRampPalette
 #' @importFrom graphics plot
+#' @importFrom graphics par
+#' @importFrom graphics text
 #' @importFrom stats cmdscale
 #' @importFrom stats loadings
 #' @importFrom stats princomp
 #' @importFrom stats varimax
 #' @importFrom utils combn
 #' 
-plot_wordlist <- function(x,connect.lines=0,
+plot_wordlist <- function(x,connect.lines="all",
                           method="PCA",dims=3,
-                            axes=F,box=F,cex=1,
-                            alpha=.5, col="black", 
-                            tvectors=tvectors,breakdown=FALSE,
-                            ...){
+                          axes=F,box=F,cex=1,legend=T, size = c(800,800),
+                          alpha="graded", alpha.grade = 1, col="rainbow", 
+                          tvectors=tvectors,breakdown=FALSE,
+                          ...){
   
   ### Compute neighbors
   
@@ -39,7 +50,7 @@ plot_wordlist <- function(x,connect.lines=0,
       message("Note: x converted to character")
     }
     
-
+    
     if(class(x) == "character"){
       
       if(breakdown==TRUE){satz1 <- breakdown(x)} 
@@ -70,7 +81,7 @@ plot_wordlist <- function(x,connect.lines=0,
       
     }
     
-   
+    
     ## Reduce dimensions
     
     if(method=="PCA"){
@@ -98,10 +109,10 @@ plot_wordlist <- function(x,connect.lines=0,
     ## Plot 2d
     
     if(dims == 2){
-    plot(Lt$x,Lt$y,xlab="Dimension 1",ylab="Dimension 2",pch=20,type="n",
-         xlim=c(min(Lt$x)-0.1,max(Lt$x)+0.1),ylim=c(min(Lt$y)-0.1,max(Lt$y)+0.1))
-    with(Lt,points(x,y,cex=.6,pch=20))
-    with(Lt,text(x,y,words2,cex=cex))
+      plot(Lt$x,Lt$y,xlab="Dimension 1",ylab="Dimension 2",pch=20,type="n",
+           xlim=c(min(Lt$x)-0.1,max(Lt$x)+0.1),ylim=c(min(Lt$y)-0.1,max(Lt$y)+0.1))
+      with(Lt,points(x,y,cex=.6,pch=20))
+      with(Lt,text(x,y,words2,cex=cex))
     }
     
     
@@ -109,6 +120,8 @@ plot_wordlist <- function(x,connect.lines=0,
     
     
     if(dims == 3){
+      par3d(windowRect = c(20, 30, size[1] + 20, size[2] + 30))
+      
       with(Lt,plot3d(x,y,z,box=box,axes=axes,xlab="",ylab="",zlab=""
                      ,xakt="n",yakt="n",zakt="n",col="black",...))
       
@@ -117,16 +130,36 @@ plot_wordlist <- function(x,connect.lines=0,
       with(Lt,text3d(x,y,z,words2))
       
       
+      ### alpha="shade" backwards compatibility
+      if(alpha=="shade"){alpha <- "graded"}
       
-      ### Rainbow palette
+      
+      ### Colour palette
       
       palette <- rainbow(101,start=0,end=0.95)
       
       
+      if(col[1] == "heat.colors"){palette <- heat.colors(101)}
+      if(col[1] == "terrain.colors"){palette <- terrain.colors(101)}
+      if(col[1] == "topo.colors"){palette <- topo.colors(101)}
+      if(col[1] == "cm.colors"){palette <- topo.colors(101)}
+      
+      if(length(col) == 2){
+        col <- rep(col,each=2)
+      }
+      
+      if(length(col) >= 3){
+        palette <- colorRampPalette(col)(101)
+      }
+      
       
       ### connect.lines : X nearest
       ### connect.lines use argument [2] from alpha and col!
       
+      if(is.numeric(connect.lines) & connect.lines > (nrow(Lt) -1)){
+        stop("cannot plot more connecting lines than number of points minus one")
+      }
+      
       if(connect.lines == "all"){
         
         
@@ -155,21 +188,44 @@ plot_wordlist <- function(x,connect.lines=0,
         
         #### Plot lines
         
-        if(col != "rainbow"){suppressWarnings(segments3d(segm,alpha=(segm$pwsim)^2))}
+        if(alpha == "graded"){
+          if(!(col[1] %in% c("rainbow","heat.colors","terrain.colors","topo.colors","cm.colors")  | length(col) >= 3)){
+            suppressWarnings(segments3d(segm,alpha=alpha.grade*(segm$pwsim)^2,col=col))}
+          
+          if(col[1] %in% c("rainbow","heat.colors","terrain.colors","topo.colors","cm.colors") | length(col) >= 3){
+            suppressWarnings(segments3d(segm,alpha=alpha.grade*(segm$pwsim)^2,col=segm$colour))}
+        }
         
-        if(col == "rainbow"){suppressWarnings(segments3d(segm,alpha=(segm$pwsim)^2,col=segm$colour))}
+        if(is.numeric(alpha)){
+          if(!(col[1] %in% c("rainbow","heat.colors","terrain.colors","topo.colors","cm.colors") | length(col) >= 3)){
+            suppressWarnings(segments3d(segm,alpha=alpha,col=col))}
+          
+          if(col[1] %in% c("rainbow","heat.colors","terrain.colors","topo.colors","cm.colors") | length(col) >= 3){
+            suppressWarnings(segments3d(segm,alpha=alpha,col=segm$colour))}
+        }
         
       }
       
+      #### Legend
       
-      
-      
-      if(length(alpha)==1){alpha <- rep(alpha,2)}
-      if(length(col)==1 && col != "rainbow"){col <- rep(col,2)}
-      
-      
-      
-      
+      if(legend == T){
+        
+        bgplot3d({
+          par(mar=c(0,0,0,0))
+          par(oma=c(0,0,0,0))
+          
+          if(col[1] %in% c("rainbow","heat.colors","terrain.colors","topo.colors","cm.colors") | length(col) >= 3){
+            legend_image <- as.raster(matrix(rev(palette), ncol=1))
+          }
+          if(!(col[1] %in% c("rainbow","heat.colors","terrain.colors","topo.colors","cm.colors") | length(col) >= 3)){
+            legend_image <- as.raster(matrix(rep(col[1],101), ncol=1))
+          }
+          plot(c(0,2),c(0,1),type = 'n', axes = F,xlab = '', ylab = '')
+          text(x=0.3, y = seq(0.1,.35,l=5), labels = seq(0,1,l=5),cex=1)
+          rasterImage(legend_image, 0.1, 0.1, 0.22,0.35)
+          text(0.18,0.05,"cosine similarity")
+        })
+      }
       
       if(class(connect.lines) == "numeric" && connect.lines > 0){
         
@@ -177,11 +233,11 @@ plot_wordlist <- function(x,connect.lines=0,
         
         which.indices       <- t(apply(cos.near, 1, order, decreasing = T)[ 1:(connect.lines+1), ])
         which.indices       <- as.data.frame(which.indices[,-1])
-        which.indices[,3:4] <- 1:n
-        which.indices       <- which.indices[,c(3,1,4,2)]
         
-        indices       <- as.vector(t(which.indices))
+        pre           <- as.vector(t(which.indices))
+        alternate     <- rep((1:n),each=connect.lines)
         
+        indices       <- c(rbind(alternate,pre))
         
         ### Prepare Segments
         
@@ -209,18 +265,22 @@ plot_wordlist <- function(x,connect.lines=0,
         
         ### Plot
         
-        if(col[1] != "rainbow" && alpha[1] != "shade"){
-          suppressWarnings(segments3d(segm,alpha=alpha[2],col=col[2]))
+        #### Plot lines
+        
+        if(alpha == "graded"){
+          if(!(col[1] %in% c("rainbow","heat.colors","terrain.colors","topo.colors","cm.colors") | length(col) >= 3)){
+            suppressWarnings(segments3d(segm,alpha=alpha.grade*(segm$pwsim)^2,col=col))}
+          
+          if(col[1] %in% c("rainbow","heat.colors","terrain.colors","topo.colors","cm.colors") | length(col) >= 3){
+            suppressWarnings(segments3d(segm,alpha=alpha.grade*(segm$pwsim)^2,col=segm$colour))}
         }
         
-        if(col[1] != "rainbow" && alpha[1] == "shade"){
-          suppressWarnings(segments3d(segm,alpha=(segm$pwsim)^2,col=col[2]))
-        }
-        
-        if(col[1] == "rainbow"){
-          
-          suppressWarnings(segments3d(segm,alpha=(segm$pwsim)^2,col=segm$colour))
+        if(is.numeric(alpha)){
+          if(!(col[1] %in% c("rainbow","heat.colors","terrain.colors","topo.colors","cm.colors") | length(col) >= 3)){
+            suppressWarnings(segments3d(segm,alpha=alpha,col=col))}
           
+          if(col[1] %in% c("rainbow","heat.colors","terrain.colors","topo.colors","cm.colors") | length(col) >= 3){
+            suppressWarnings(segments3d(segm,alpha=alpha,col=segm$colour))}
         }
         
       }    
@@ -228,11 +288,6 @@ plot_wordlist <- function(x,connect.lines=0,
       
       
       
-      ### lines from original word to vectors
-      ### start.lines use argument [1] from alpha and col!
-      
-      
-          
     }
     
     Lt[,-which(colnames(Lt) %in% c("words","words2"))]
